//
// Created by Иван_Архипов on 23.11.2017.
//
#define _CRT_SECURE_NO_WARNINGS
#include <iostream>
#include <ctime>
#include <algorithm>

#ifdef WIN32
#include <direct.h>
#define mkdir(dir, mode) _mkdir(dir)
#endif

#include "LotroDat.h"
using namespace LOTRO_DAT;
using namespace std;

int main() {
    std::cout << "Gi1dor's LotRO .dat patcher ver. 2.1.0" << std::endl;
    freopen("errors.log", "w", stderr);

   // setbuf(stdout, NULL);
    setbuf(stderr, NULL);

   // setvbuf (stdout, NULL, _IONBF, BUFSIZ);
    setvbuf (stderr, NULL, _IONBF, BUFSIZ);

    try {
        std::cout << "Hello! I'm a basic shell version of .dat file patcher.\n";
        DatFile file;
        
        ifstream in("dat_file_path.txt");
        if (!in.fail()) {
            std::string filename;
            getline(in, filename);
            try {
                std::cout << "Using .dat file from dat_file_path.txt...\n";
                std::cout << "Opening file " << filename << std::endl;
                file.InitDatFile(filename.c_str(), 0);
            } catch (DatException &e) {
                std::cout << "Dat file path from dat_file_path.txt - " << filename << " may be incorrect (cannot open DatFile there)\n";
            }
        }

        while (file.DatFileState() == CLOSED) {
            std::cout << "Please, tell, where the .dat file is\n";
            std::cout << "Enter path to file (including filename): ";
            std::string filename;
            std::getline(std::cin, filename);

            std::cout << "Opening file " << filename << std::endl;

            try {
                file.InitDatFile(filename.c_str(), 0);
            } catch (DatException &e) {
                if (e.type() == NOFILE_EXCEPTION)
                    std::cout << "Cannot find file... Could you enter .dat filename once more?" << std::endl;
                else
                    std::cout << "Some error caused while opening the file... "
                            "Could you enter .dat filename once more?" << std::endl;
                file.CloseDatFile();
            }
        }

        std::cout << "Great! File initialised successfully!\n";
        std::cout << "Files number: " << file.files_number() << std::endl;

        while (true) {
            std::cout << "Please, choose, what should I do. I can patch datfile from database to .dat file (enter 1), "
                    "change locale (enter 2), print current locale (enter 3) or exit (enter -1)\n";
            int cmd = 0;
            std::cout << "Enter number of command (1-4): ";
            std::cin >> cmd;

            std::string tmp;
            std::getline(std::cin, tmp);

            if (cmd == -1) {
                std::cout << "Exiting. Thanks for using me!\n";
                break;
            }
            if (cmd == 1) {
                std::cout << "You've chosen to patch database! Write name of database file (it should be in the same "
                        "directory), of enter 0 to return to main dialogue.\n";
                while (true) {
                    std::cout << "Enter name of file or 0: ";
                    std::string dbname;
                    std::getline(std::cin, dbname);

                    if (dbname == std::to_string(0)) {
                        std::cout << "Okay, returning back...\n\n";
                        break;
                    }

                    Database db;
                    try {
                        std::cout << "Opening database... " << dbname << std::endl;
                        db.InitDatabase(dbname);
                        if (db.CountRows() == 0) {
                            std::cout << "There are no files in database or database doesn't exist. "
                                    "Please, try again!\n";
                            continue;
                        }
                        std::cout << "There are " << db.CountRows() << " files in database." << std::endl;
                    } catch (DatException &e) {
                        std::cout << "Unfortunately, I cannot open this database. Could you try again please?\n";
                        continue;
                    }

                    std::cout << "Successfully opened database! Beginning patching...\n";
                    const clock_t begin_time = clock();
                    size_t all = db.CountRows();
                    size_t now = 0;

                    SubfileData subfile;

                    try {
                        subfile = db.GetNextFile();
                    } catch (std::exception &e) {
                        fprintf(stderr, "Caught %s exception.\n", e.what());
                        fprintf(stderr, "DatFile::PatchAllDatabase() error! Caught exception while fetching "
                                "file from database! Stopping...\n");
                        return false;
                    }
                    while (!subfile.Empty()) {
                        try {
                            file.PatchFile(subfile);
                        } catch (std::exception &e) {
                            fprintf(stderr, "Caught %s exception.\n", e.what());
                            fprintf(stderr, "DatFile::PatchAllDatabase() error! Caught exception while "
                                    "patching file! Passing...\n");
                        }

                        try {
                            subfile = db.GetNextFile();
                        } catch (std::exception &e) {
                            fprintf(stderr, "Caught %s exception.\n", e.what());
                            fprintf(stderr,"DatFile::PatchAllDatabase() error! Caught exception while fetching "
                                            "file from database! Stopping...\n");
                            break;
                        }
                        ++now;
                        if (now * 100 / all > (now - 1) * 100 / all)
                            std::cout << now * 100 / all << "%\n";
                    }

                    db.CloseDatabase();
                    file.CommitChanges();

                    fprintf(stdout, "Spent %f seconds on patching! Thank you for your patience!\n",
                            float(clock() - begin_time) / CLOCKS_PER_SEC);

                    std::cout << "Great! File was patched successfully!\n\n";
                    break;
                }
            }
            if (cmd == 2) {
                std::cout << "Old locale is " << (file.current_locale() == PATCHED ? "RU" : "Original") << endl;
                std::cout << "Changing locale..." << std::endl;
                file.SetLocale(file.current_locale() == PATCHED ? ORIGINAL : PATCHED);
                std::cout << "New locale is " << (file.current_locale() == PATCHED ? "RU" : "Original") << endl << endl;
            }

            if (cmd == 3) {
                std::cout << "Current locale is " << (file.current_locale() == PATCHED ? "RU" : "Original") << endl << endl;
            }
        }
        file.CloseDatFile();
    } catch (std::exception &e) {
        fprintf(stderr, "Caught %s exception.", e.what());
        printf("Caught %s exception.", e.what());
        fflush(stdout);

        printf("Some critical errors occured. Need to stop execution. See information in errors.log file\n");
        fprintf(stderr, "Some critical errors occured. Need to stop execution now...\n");
    }

    system("pause");
    return 0;
}

